"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const meta_data_1 = require("./meta_data");
const util_1 = require("./util");
function serializeUsing(serializer, keyName) {
    return function (target, actualKeyName) {
        const metadata = meta_data_1.MetaData.getMetaData(target.constructor, actualKeyName);
        metadata.serializedKey = (keyName) ? keyName : actualKeyName;
        metadata.serializedType = serializer;
        metadata.flags |= 4096;
    };
}
exports.serializeUsing = serializeUsing;
function serializeAs(type, keyName) {
    return function (target, actualKeyName) {
        const metadata = meta_data_1.MetaData.getMetaData(target.constructor, actualKeyName);
        metadata.serializedKey = (keyName) ? keyName : actualKeyName;
        metadata.serializedType = type;
        metadata.flags |= 16384;
        metadata.flags = util_1.setBitConditionally(metadata.flags, 4, util_1.isPrimitiveType(type));
    };
}
exports.serializeAs = serializeAs;
function serializeAsArray(type, keyName) {
    return function (target, actualKeyName) {
        const metadata = meta_data_1.MetaData.getMetaData(target.constructor, actualKeyName);
        metadata.serializedKey = (keyName) ? keyName : actualKeyName;
        metadata.serializedType = type;
        metadata.flags |= 16;
        metadata.flags = util_1.setBitConditionally(metadata.flags, 4, util_1.isPrimitiveType(type));
    };
}
exports.serializeAsArray = serializeAsArray;
function serializeAsMap(type, keyName) {
    return function (target, actualKeyName) {
        const metadata = meta_data_1.MetaData.getMetaData(target.constructor, actualKeyName);
        metadata.serializedKey = (keyName) ? keyName : actualKeyName;
        metadata.serializedType = type;
        metadata.flags |= 64;
        metadata.flags = util_1.setBitConditionally(metadata.flags, 4, util_1.isPrimitiveType(type));
    };
}
exports.serializeAsMap = serializeAsMap;
function serializeAsJson(keyNameOrTransformKeys, transformKeys = true) {
    return function (target, actualKeyName) {
        const metadata = meta_data_1.MetaData.getMetaData(target.constructor, actualKeyName);
        metadata.serializedKey = (typeof keyNameOrTransformKeys === "string") ? keyNameOrTransformKeys : actualKeyName;
        metadata.flags |= (256);
        const shouldTransformKeys = typeof keyNameOrTransformKeys === "boolean" ? keyNameOrTransformKeys : transformKeys;
        metadata.flags = util_1.setBitConditionally(metadata.flags, 1024, shouldTransformKeys);
    };
}
exports.serializeAsJson = serializeAsJson;
function deserializeUsing(serializer, keyName) {
    return function (target, actualKeyName) {
        const metadata = meta_data_1.MetaData.getMetaData(target.constructor, actualKeyName);
        metadata.deserializedKey = (keyName) ? keyName : actualKeyName;
        metadata.deserializedType = serializer;
        metadata.flags |= 2048;
    };
}
exports.deserializeUsing = deserializeUsing;
function deserializeAs(type, keyName) {
    return function (target, actualKeyName) {
        const metadata = meta_data_1.MetaData.getMetaData(target.constructor, actualKeyName);
        metadata.deserializedKey = (keyName) ? keyName : actualKeyName;
        metadata.deserializedType = type;
        metadata.flags |= 8192;
        metadata.flags = util_1.setBitConditionally(metadata.flags, 2, util_1.isPrimitiveType(type));
    };
}
exports.deserializeAs = deserializeAs;
function deserializeAsArray(type, keyName) {
    return function (target, actualKeyName) {
        const metadata = meta_data_1.MetaData.getMetaData(target.constructor, actualKeyName);
        metadata.deserializedKey = (keyName) ? keyName : actualKeyName;
        metadata.deserializedType = type;
        metadata.flags |= 8;
        metadata.flags = util_1.setBitConditionally(metadata.flags, 2, util_1.isPrimitiveType(type));
    };
}
exports.deserializeAsArray = deserializeAsArray;
function deserializeAsMap(type, keyName) {
    return function (target, actualKeyName) {
        const metadata = meta_data_1.MetaData.getMetaData(target.constructor, actualKeyName);
        metadata.deserializedKey = (keyName) ? keyName : actualKeyName;
        metadata.deserializedType = type;
        metadata.flags |= 32;
        metadata.flags = util_1.setBitConditionally(metadata.flags, 2, util_1.isPrimitiveType(type));
    };
}
exports.deserializeAsMap = deserializeAsMap;
function deserializeAsJson(keyNameOrTransformKeys, transformKeys = true) {
    return function (target, actualKeyName) {
        const metadata = meta_data_1.MetaData.getMetaData(target.constructor, actualKeyName);
        metadata.deserializedKey = (typeof keyNameOrTransformKeys === "string") ? keyNameOrTransformKeys : actualKeyName;
        metadata.flags |= (128);
        const shouldTransformKeys = typeof keyNameOrTransformKeys === "boolean" ? keyNameOrTransformKeys : transformKeys;
        metadata.flags = util_1.setBitConditionally(metadata.flags, 512, shouldTransformKeys);
    };
}
exports.deserializeAsJson = deserializeAsJson;
function autoserializeUsing(serializer, keyName) {
    return function (target, actualKeyName) {
        const metadata = meta_data_1.MetaData.getMetaData(target.constructor, actualKeyName);
        const key = (keyName) ? keyName : actualKeyName;
        metadata.serializedKey = key;
        metadata.deserializedKey = key;
        metadata.serializedType = serializer.Serialize;
        metadata.deserializedType = serializer.Deserialize;
        metadata.flags |= 6144;
    };
}
exports.autoserializeUsing = autoserializeUsing;
function autoserializeAs(type, keyName) {
    return function (target, actualKeyName) {
        const metadata = meta_data_1.MetaData.getMetaData(target.constructor, actualKeyName);
        const key = (keyName) ? keyName : actualKeyName;
        metadata.deserializedKey = key;
        metadata.serializedKey = key;
        metadata.deserializedType = type;
        metadata.serializedType = type;
        metadata.flags |= (16384 | 8192);
        metadata.flags = util_1.setBitConditionally(metadata.flags, 6, util_1.isPrimitiveType(type));
    };
}
exports.autoserializeAs = autoserializeAs;
function autoserializeAsArray(type, keyName) {
    return function (target, actualKeyName) {
        const metadata = meta_data_1.MetaData.getMetaData(target.constructor, actualKeyName);
        const key = (keyName) ? keyName : actualKeyName;
        metadata.deserializedKey = key;
        metadata.serializedKey = key;
        metadata.deserializedType = type;
        metadata.serializedType = type;
        metadata.flags |= (16 | 8);
        metadata.flags = util_1.setBitConditionally(metadata.flags, 6, util_1.isPrimitiveType(type));
    };
}
exports.autoserializeAsArray = autoserializeAsArray;
function autoserializeAsMap(type, keyName) {
    return function (target, actualKeyName) {
        const metadata = meta_data_1.MetaData.getMetaData(target.constructor, actualKeyName);
        const key = (keyName) ? keyName : actualKeyName;
        metadata.deserializedKey = key;
        metadata.serializedKey = key;
        metadata.deserializedType = type;
        metadata.serializedType = type;
        metadata.flags |= (64 | 32);
        metadata.flags = util_1.setBitConditionally(metadata.flags, 6, util_1.isPrimitiveType(type));
    };
}
exports.autoserializeAsMap = autoserializeAsMap;
function autoserializeAsJson(keyNameOrTransformKeys, transformKeys = true) {
    return function (target, actualKeyName) {
        const metadata = meta_data_1.MetaData.getMetaData(target.constructor, actualKeyName);
        const key = (typeof keyNameOrTransformKeys === "string") ? keyNameOrTransformKeys : actualKeyName;
        const shouldTransformKeys = typeof keyNameOrTransformKeys === "boolean" ? keyNameOrTransformKeys : transformKeys;
        metadata.deserializedKey = key;
        metadata.serializedKey = key;
        metadata.flags |= (256 | 128);
        metadata.flags = util_1.setBitConditionally(metadata.flags, 1536, shouldTransformKeys);
    };
}
exports.autoserializeAsJson = autoserializeAsJson;
function inheritSerialization(parentType) {
    return function (childType) {
        meta_data_1.MetaData.inheritMetaData(parentType, childType);
    };
}
exports.inheritSerialization = inheritSerialization;
//# sourceMappingURL=annotations.js.map