"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const util_1 = require("./util");
const string_transforms_1 = require("./string_transforms");
const TypeMap = new Map();
class MetaData {
    constructor(keyName) {
        this.keyName = keyName;
        this.serializedKey = "";
        this.deserializedKey = "";
        this.deserializedType = Function;
        this.serializedType = Function;
        this.flags = 0;
    }
    getSerializedKey() {
        if (this.serializedKey === this.keyName) {
            return MetaData.serializeKeyTransform(this.keyName);
        }
        return this.serializedKey ? this.serializedKey : this.keyName;
    }
    getDeserializedKey() {
        if (this.deserializedKey === this.keyName) {
            return MetaData.deserializeKeyTransform(this.keyName);
        }
        return MetaData.deserializeKeyTransform(this.deserializedKey ? this.deserializedKey : this.keyName);
    }
    static hasKeyName(metadataArray, key) {
        for (var i = 0; i < metadataArray.length; i++) {
            if (metadataArray[i].keyName === key)
                return true;
        }
        return false;
    }
    static clone(data) {
        const metadata = new MetaData(data.keyName);
        metadata.deserializedKey = data.deserializedKey;
        metadata.serializedKey = data.serializedKey;
        metadata.serializedType = data.serializedType;
        metadata.deserializedType = data.deserializedType;
        metadata.flags = data.flags;
        return metadata;
    }
    static getMetaData(target, keyName) {
        var metaDataList = TypeMap.get(target);
        if (metaDataList === void 0) {
            metaDataList = [];
            TypeMap.set(target, metaDataList);
        }
        for (var i = 0; i < metaDataList.length; i++) {
            if (metaDataList[i].keyName === keyName) {
                return metaDataList[i];
            }
        }
        metaDataList.push(new MetaData(keyName));
        return metaDataList[metaDataList.length - 1];
    }
    static inheritMetaData(parentType, childType) {
        var parentMetaData = TypeMap.get(parentType) || [];
        var childMetaData = TypeMap.get(childType) || [];
        for (var i = 0; i < parentMetaData.length; i++) {
            const keyName = parentMetaData[i].keyName;
            if (!MetaData.hasKeyName(childMetaData, keyName)) {
                childMetaData.push(MetaData.clone(parentMetaData[i]));
            }
        }
        TypeMap.set(childType, childMetaData);
    }
    static getMetaDataForType(type) {
        if (type !== null && type !== void 0) {
            return TypeMap.get(type) || null;
        }
        return null;
    }
}
MetaData.TypeMap = TypeMap;
MetaData.serializeKeyTransform = string_transforms_1.NoOp;
MetaData.deserializeKeyTransform = string_transforms_1.NoOp;
MetaData.deserializeInstantationMethod = util_1.InstantiationMethod.New;
exports.MetaData = MetaData;
//# sourceMappingURL=meta_data.js.map