"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const util_1 = require("./util");
const meta_data_1 = require("./meta_data");
function SerializeMap(source, type) {
    const target = {};
    const keys = Object.keys(source);
    for (let i = 0; i < keys.length; i++) {
        const key = keys[i];
        const value = source[key];
        if (value !== void 0) {
            target[meta_data_1.MetaData.serializeKeyTransform(key)] = Serialize(value, type);
        }
    }
    return target;
}
exports.SerializeMap = SerializeMap;
function SerializeArray(source, type) {
    const retn = new Array(source.length);
    for (let i = 0; i < source.length; i++) {
        retn[i] = Serialize(source[i], type);
    }
    return retn;
}
exports.SerializeArray = SerializeArray;
function SerializePrimitive(source, type) {
    if (source === null || source === void 0) {
        return null;
    }
    if (type === String)
        return source.toString();
    if (type === Boolean)
        return Boolean(source);
    if (type === Number) {
        const value = Number(source);
        if (isNaN(value))
            return null;
        return value;
    }
    if (type === Date)
        return source.toString();
    if (type === RegExp)
        return source.toString();
    return source.toString();
}
exports.SerializePrimitive = SerializePrimitive;
function SerializeJSON(source, transformKeys = true) {
    if (source === null || source === void 0)
        return null;
    if (Array.isArray(source)) {
        const array = new Array(source.length);
        for (let i = 0; i < source.length; i++) {
            array[i] = SerializeJSON(source[i], transformKeys);
        }
        return array;
    }
    const type = typeof source;
    if (type === "object") {
        if (source instanceof Date || source instanceof RegExp) {
            return source.toString();
        }
        else {
            const retn = {};
            const keys = Object.keys(source);
            for (let i = 0; i < keys.length; i++) {
                const key = keys[i];
                const value = source[key];
                if (value !== void 0) {
                    const retnKey = transformKeys ? meta_data_1.MetaData.serializeKeyTransform(key) : key;
                    retn[retnKey] = SerializeJSON(value, transformKeys);
                }
            }
            return retn;
        }
    }
    else if (type === "function") {
        return null;
    }
    return source;
}
exports.SerializeJSON = SerializeJSON;
function Serialize(instance, type) {
    if (instance === void 0 || instance === null) {
        return null;
    }
    const metadataList = meta_data_1.MetaData.getMetaDataForType(type);
    if (metadataList === null) {
        if (util_1.isPrimitiveType(type)) {
            return SerializePrimitive(instance, type);
        }
        else {
            return {};
        }
    }
    const target = {};
    for (let i = 0; i < metadataList.length; i++) {
        const metadata = metadataList[i];
        if (metadata.serializedKey === null)
            continue;
        const source = instance[metadata.keyName];
        if (source === void 0)
            continue;
        const keyName = metadata.getSerializedKey();
        const flags = metadata.flags;
        if ((flags & 64) !== 0) {
            target[keyName] = SerializeMap(source, metadata.serializedType);
        }
        else if ((flags & 16) !== 0) {
            target[keyName] = SerializeArray(source, metadata.serializedType);
        }
        else if ((flags & 4) !== 0) {
            target[keyName] = SerializePrimitive(source, metadata.serializedType);
        }
        else if ((flags & 16384) !== 0) {
            target[keyName] = Serialize(source, metadata.serializedType);
        }
        else if ((flags & 256) !== 0) {
            target[keyName] = SerializeJSON(source, (flags & 1024) !== 0);
        }
        else if ((flags & 4096) !== 0) {
            target[keyName] = metadata.serializedType(source);
        }
    }
    if (typeof type.onSerialized === "function") {
        const value = type.onSerialized(target, instance);
        if (value !== void 0) {
            return value;
        }
    }
    return target;
}
exports.Serialize = Serialize;
//# sourceMappingURL=serialize.js.map